{
 ==================================================================================
 TIndentListBox
 INDENTED LISTBOX COMPONENT V 1.2 Oct-96
 By Santiago Portela:  sportela@cece.es
 Please feel free to copy, modify or distribute this component
 ==================================================================================
 This component displays multiline items in a list box.


 TlistBox Descendant. 6 new properties:
   indent
   PosIndent
   DrawLines
   DrawLineColor
   TitleFontColor
   TitleFontStyle


 indent property
       inone          wraps each item to the number of lines required.

       itab           split each line where '|' (#124) is found. First part
                      is displayed aligned to the leftmost part of the component;
                      second part is displayed left aligned to the indent position
                      (the indent position is set in property PosIndent).
                      Text is wrapped to the number of lines required.

       iline          Same as itab, but instead of spliting vertically the
                      component, first part is displayed in first line and
                      second part in the following lines.

                      Default is inone

 PosIndent property
       integer        Used only when Indent is itab. PosIndent is the position
                      (in pixels) inside the control where second part of
                      itms are left aligned.

                      Default is 48

 DrawLines property
       boolean        Used with itab, iline. If TRUE, a line is drawed below
                      each item.

DrawLineColor property
       Tcolor         Color of lines if DrawLine is TRUE.

TitleFontColor property
       TColor         Used with itab, iline. Color of the font of first part.

TitleFontStyle property
    TFontStyles       Used with itab, iline. Style of the font of first part.
                      Note: why not to use a font?- Because of font's heights.



 *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS* *BUGS*
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

 1.If component's font is changed, please refill items list, otherwise the
   height of lines is still calculated upon previous font.

 2.VERY CAREFULL setting width too small. Component may hang up.

 ==================================================================================
}

unit Il;

interface


uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

  type
  TEnumType = (inone,itab,iline);

type
  TIndentListBox = class(TListBox)
  private

    Findent:TEnumType;                    {--- indent option---}
    Fi:integer;                           {--- indent position--}
    lista:tstringlist;                    {--- working list----}
    alto:integer;                         {--default char height--}
    TFColor:tcolor;                      {--title font color&style}
    LineColor:TColor;
    TFStyle:TFontStyles;
    DoDrawLines:boolean;                  {-- wether draw lines or not--}
  protected
    Procedure SetIndent(i:TEnumType);     {-- sets Findent---}
    Function  GetIndent:TEnumType;        {-- retrieves Findent--}
    Procedure SetFi(i:integer);           {--sets Fi--}
    Function  GetFi:integer;              {--retrieves Fi--}

    Procedure DrawItem(Index:integer;R:trect;State:TownerDrawState);override;
    {-- Custom drawing of items in the list---}
    Procedure MeasureItem(Index:integer;var altura:integer);override;

    {-- Callback to calculate each item height in the list--}
  public
    constructor Create(owner:tcomponent);override;
    destructor Destroy;override;
  published
    property Indent:TenumType read GetIndent write SetIndent ;
    Property DrawLines:boolean read DoDrawLines write DoDrawLines;
    Property DrawLineColor:TColor read LineColor write LineColor;
    property PosIndent:integer read GetFi write SetFi;
    property TitleFontColor:TColor read TFColor write TFColor;
    property TitleFontStyle:TFontStyles read TFStyle write TFStyle;
  end;

procedure Register;

implementation

{---------------------------------------------------------------------------}
Procedure rtrim(var ax:string);  { rtrim eliminates training whitespaces at the end of the string}
var i:integer;
begin
 i:=length(ax);
 while((i>0) and (ax[i]=' ')) do dec(i);
{ ax[0]:=chr(i);}
 ax:=copy(ax,1,i);
end;


Procedure wrapx(ax:string;tl:tstrings;can:tcanvas;n:integer);
var k,m:integer;
    bx,cx:string;
    hecho:boolean;
begin
rtrim(ax);
hecho:=false;
 tl.clear;
 repeat
 k:=can.textwidth(ax);
 if k<=n then begin
   tl.add(ax);
   hecho:=true;
 end
 else begin
   m:=1;
   bx:=ax[m];
   while (can.textwidth(bx)<n) do begin
     inc(m);
     bx:=bx+ax[m];
   end;

   bx:=copy(ax,1,m);
   cx:=copy(ax,m+1,length(ax)-m);
   while ((bx[m]<> ' ') and (can.textwidth(copy(bx,1,m)) < n)) do begin
    cx:=bx[m]+cx;
    dec(m);

   end;

   {bx[0]:=chr(m);}
   bx:=copy(bx,1,m);

   tl.add(bx);
   ax:=cx;
 end;
 until hecho;

end;


{----------------------------------------------------------------------------}


Constructor TIndentListBox.Create(owner:tcomponent);
begin
    inherited create(owner);
    Left := 20;
    Top  := 20;
    Width := 100;
    Height := 100;
    Font.Color := clBlack;
    Font.Height := -11;
    Font.Name := 'Arial';
    Font.Style := [];
    ItemHeight := 16;
    ParentFont := False;
    Style := lbOwnerDrawVariable;
    TFColor:=clBlack;
    TFStyle:=[fsItalic];
    Indent := itab;
    PosIndent := 48;
    DrawLines := True;
    DrawLineColor:=clSilver;

    lista:=tstringlist.create;

end;

destructor TIndentListBox.Destroy;
begin
    lista.clear;
    lista.destroy;
    inherited destroy;
end;



Procedure TIndentListBox.SetIndent(i:TEnumType);
begin
  if Findent<>i then begin
      Findent:=i;
      Style := lbOwnerDrawVariable;
      invalidate;
  end;
end;

Function  TIndentListBox.GetIndent:TEnumType;
begin
  GetIndent:=Findent;
end;

Procedure TIndentListBox.SetFi(i:integer);
begin
 if Fi<>i then begin
     Fi:=i;
     invalidate;
 end;
end;

Function TIndentListBox.GetFi:integer;
begin
  GetFi:=Fi;
end;



Procedure TIndentListBox.MeasureItem(Index:integer;var altura:integer);
var
    nlines:integer;
    ax,bx:string;
    postab:integer;

begin

  inherited MeasureItem(Index,altura);
  postab:=pos(#124,items[index]);
  if postab>0 then begin
    ax:=copy(items[index],1,postab-1);
    bx:=copy(items[index],postab+1,length(items[index])-postab);
  end
  else begin
    ax:=' ';
    bx:=items[index];
  end;
  canvas.font:=font;
  with canvas do begin
      if width<=posindent then posindent:=width div 2;
      nlines:=1;
      alto:=canvas.textheight('|_M');
      case indent of
       inone:begin
	    wrapx(items[index],lista,canvas,width-20);
            nlines:=lista.count;
         end;
       itab:begin
            wrapx(bx,lista,canvas,width-posindent-20);
            nlines:=lista.count;
         end;
       iline:begin
            wrapx(bx,lista,canvas,width-20);
            nlines:=lista.count+1;
         end;
      end;
  end;
  altura:=nlines*alto;
end;

Procedure TIndentListBox.DrawItem(Index:integer;R:trect;State:TownerDrawState);
var
  ax,bx:string;
  postab,i:integer;
  oldf:tfont;
begin

  oldf:=tfont.create;
  oldf.assign(canvas.font);
  postab:=pos(#124,items[index]);
  if postab>0 then begin
    ax:=copy(items[index],1,postab-1);
    bx:=copy(items[index],postab+1,length(items[index])-postab);
  end
  else begin
    ax:=' ';
    bx:=items[index];
  end;

  with Canvas do	{ draw on the control canvas, not on the form }
  begin
    FillRect(R);	{ clear the rectangle }
    case indent of
     inone:begin
                 wrapx(items[index],lista,canvas,width-20);
                 for i:=0 to lista.count-1 do
                 TextOut(R.Left +1, R.Top+i*alto, lista.strings[i]);
           end;
     itab:begin
            wrapx(bx,lista,canvas,width-posindent-20);
            lista.add(ax);
            canvas.font.color:=TFColor;canvas.font.style:=TFStyle;
                 TextOut(r.left,r.top,lista.strings[lista.count-1]);
            canvas.font.assign(Oldf);
                 for i:=0 to lista.count-2 do
                 TextOut(R.Left +posindent, R.Top+i*alto, lista.strings[i]);
            if DoDrawLines then begin
               canvas.pen.color:=DrawLineColor;
               MoveTo(R.Left , R.Top+(lista.count-1)*alto-1);
               LineTo(R.right, R.Top+(lista.count-1)*alto-1);
            end;
          end;
      iline:begin
            wrapx(bx,lista,canvas,width-20);
            lista.add(ax);
            canvas.font.color:=TFColor;canvas.font.style:=TFStyle;
                 TextOut(r.left,r.top,lista.strings[lista.count-1]);
            canvas.font.assign(Oldf);
                 for i:=0 to lista.count-2 do
                 TextOut(R.Left , R.Top+(i+1)*alto, lista.strings[i]);
            if DoDrawLines then begin
               canvas.pen.color:=DrawLineColor;
               MoveTo(R.Left , R.Top+(lista.count)*alto-1);
               LineTo(R.right, R.Top+(lista.count)*alto-1);
            end;

            end;
    end;
  end;

  oldf.destroy;
end;




procedure Register;
begin
  RegisterComponents('SDS Tools', [TIndentListBox]);
end;

end.
